/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.ui.util;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.ui.ISelectionService;
import org.eclipse.ui.PlatformUI;

/**
 * Utility methods for {@link EObject} selections.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating GREEN Hash: 5E416E2DE253148BB65FF09E09760B23
 */
public final class EObjectSelectionUtils {

	/**
	 * Returns the first selected element if it is an EObject or <code>null</code> otherwise.
	 * 
	 * @return the first selected {@link EObject}
	 */
	public static EObject getCurrentSelectionFirstElement() {
		return getFirstElement(getCurrentSelection());
	}

	/**
	 * Returns the list of selected elements filtered for EObjects.
	 * 
	 * @return the filtered list of selected elements
	 */
	public static List<EObject> getCurrentSelectionEObjects() {
		return getEObjectElements(getCurrentSelection());
	}

	/**
	 * Returns the first selected EObject or <code>null</code> if no EObject is
	 * selected. If the selection implements {@link IAdaptable}, it is asked to
	 * adapt to {@link EObject}.
	 * 
	 * @param selection
	 *            the current selection to be search for an {@link EObject}
	 * @return the first {@link EObject} element in the selection
	 */
	public static EObject getFirstElement(ISelection selection) {
		if(selection instanceof IStructuredSelection && !selection.isEmpty()) {
			if(((IStructuredSelection)selection).getFirstElement() instanceof EObject) {
				return (EObject)((IStructuredSelection)selection).getFirstElement();
			}
			if(((IStructuredSelection)selection).getFirstElement() instanceof IAdaptable) {
				IAdaptable adaptable =
						(IAdaptable)((IStructuredSelection)selection).getFirstElement();
				return (EObject)adaptable.getAdapter(EObject.class);
			}
		}
		return null;
	}

	/**
	 * Returns the current selection obtained from the platform.
	 * 
	 * @return the current selection
	 */
	public static ISelection getCurrentSelection() {
		return getSelectionService().getSelection();
	}

	/**
	 * Returns the platform selection service.
	 * 
	 * @return the platform selection service
	 */
	public static ISelectionService getSelectionService() {
		return PlatformUI.getWorkbench().getActiveWorkbenchWindow().getSelectionService();
	}

	/**
	 * Extracts all {@link EObject}s from the given selection.
	 * 
	 * @param selection
	 *            the selection to be searched for {@link EObject}s
	 * @return the list of selected {@link EObject}s
	 */
	public static List<EObject> getEObjectElements(ISelection selection) {
		List<EObject> result = new ArrayList<EObject>();
		if(selection instanceof StructuredSelection) {
			for(Object o : ((StructuredSelection)selection).toList()) {
				if(o instanceof EObject) {
					result.add((EObject)o);
				}
			}
		}
		return result;
	}
}
